@@ -6,23 +6,22 @@ use My::Platform;
 sub skip_combinations {
   my @combinations;
 
-  # disable innodb/xtradb combinatons for configurations that were not built
+  # disable innodb combinations for configurations that were not built
   push @combinations, 'innodb_plugin' unless $ENV{HA_INNODB_SO};
 
-  # if something is compiled in, it's xtradb. innodb is MODULE_ONLY:
-  push @combinations, 'xtradb' unless $::mysqld_variables{'innodb'} eq "ON";
-  push @combinations, 'innodb';
+  push @combinations, 'innodb' unless $::mysqld_variables{'innodb'} eq "ON";
 
-  # XtraDB is RECOMPILE_FOR_EMBEDDED, ha_xtradb.so cannot work with embedded server
-  push @combinations, 'xtradb_plugin' if not $ENV{HA_XTRADB_SO}
-                                          or $::opt_embedded_server;
+  my %skip = ( 'include/have_innodb.combinations' => [ @combinations ]);
 
-  my %skip = ( 'include/have_innodb.combinations' => [ @combinations ],
-               'include/have_xtradb.combinations' => [ @combinations ]);
+  $skip{'include/innodb_encrypt_log.combinations'} = [ 'crypt' ]
+                unless $ENV{DEBUG_KEY_MANAGEMENT_SO};
 
   # don't run tests for the wrong platform
   $skip{'include/platform.combinations'} = [ (IS_WINDOWS) ? 'unix' : 'win' ];
 
+  $skip{'include/maybe_debug.combinations'} =
+    [ defined $::mysqld_variables{'debug-dbug'} ? 'release' : 'debug' ];
+
   # and for the wrong word size
   # check for exact values, in case the default changes to be small everywhere
   my $longsysvar= $::mysqld_variables{'max-binlog-stmt-cache-size'};
@@ -49,24 +48,25 @@ sub skip_combinations {
 
   $skip{'include/not_windows.inc'} = 'Requires not Windows' if IS_WINDOWS;
 
-  $skip{'t/plugin_loaderr.test'} = 'needs compiled-in innodb'
+  $skip{'main/plugin_loaderr.test'} = 'needs compiled-in innodb'
             unless $::mysqld_variables{'innodb'} eq "ON";
 
   # disable tests that use ipv6, if unsupported
   sub ipv6_ok() {
     use Socket;
     return 0 unless socket my $sock, PF_INET6, SOCK_STREAM, getprotobyname('tcp');
-    # eval{}, if there's no Socket::sockaddr_in6 at all, old Perl installation
-    eval { connect $sock, sockaddr_in6(7, Socket::IN6ADDR_LOOPBACK) };
-    return $@ eq "";
+    $!="";
+    # eval{}, if there's no Socket::sockaddr_in6 at all, old Perl installation <5.14
+    eval { bind $sock, sockaddr_in6($::baseport, Socket::IN6ADDR_LOOPBACK) };
+    return $@ eq "" && $! eq ""
   }
   $skip{'include/check_ipv6.inc'} = 'No IPv6' unless ipv6_ok();
 
-  $skip{'t/openssl_6975.test'} = 'no or too old openssl'
+  $skip{'main/openssl_6975.test'} = 'no or wrong openssl version'
     unless $::mysqld_variables{'version-ssl-library'} =~ /OpenSSL (\S+)/
-       and $1 ge "1.0.1d";
+       and $1 ge "1.0.1d" and $1 lt "1.1.1";
 
-  $skip{'t/ssl_7937.combinations'} = [ 'x509v3' ]
+  $skip{'main/ssl_7937.combinations'} = [ 'x509v3' ]
     unless $::mysqld_variables{'version-ssl-library'} =~ /OpenSSL (\S+)/
        and $1 ge "1.0.2";
 
@@ -74,4 +74,3 @@ sub skip_combinations {
 }
 
 bless { };
-
