@@ -9,8 +9,10 @@ return "Not run for embedded server" if $::opt_embedded_server;
 return "WSREP is not compiled in" unless defined $::mysqld_variables{'wsrep-on'};
 
 my ($provider) = grep { -f $_ } $ENV{WSREP_PROVIDER},
-                                "/usr/lib/galera/libgalera_smm.so",
-                                "/usr/lib64/galera/libgalera_smm.so";
+                                "/usr/lib64/galera-3/libgalera_smm.so",
+                                "/usr/lib64/galera/libgalera_smm.so",
+                                "/usr/lib/galera-3/libgalera_smm.so",
+                                "/usr/lib/galera/libgalera_smm.so";
 
 return "No wsrep provider library" unless -f $provider;
 
@@ -30,9 +32,12 @@ push @::global_suppressions,
      qr(WSREP: Failed to send state UUID:.*),
      qr(WSREP: wsrep_sst_receive_address.*),
      qr(WSREP: Could not open saved state file for reading: .*),
+     qr(WSREP: Could not open state file for reading: .*),
      qr(WSREP: last inactive check more than .* skipping check),
      qr(WSREP: Gap in state sequence. Need state transfer.),
      qr(WSREP: Failed to prepare for incremental state transfer: .*),
+     qr(WSREP: SYNC message from member .* in non-primary configuration. Ignored.),
+     qr|WSREP: access file\(.*gvwstate.dat\) failed\(No such file or directory\)|,
    );
 
 
