@@ -12,10 +12,10 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
 
 
-#include <my_global.h> 
+#include "mariadb.h"
 
 #ifdef HAVE_SPATIAL
 
@@ -663,6 +663,17 @@ Gcalc_operation_reducer::Gcalc_operation_reducer(size_t blk_size) :
 {}
 
 
+Gcalc_operation_reducer::Gcalc_operation_reducer(
+                           const Gcalc_operation_reducer &gor) :
+  Gcalc_dyn_list(gor),
+#ifndef GCALC_DBUG_OFF
+  n_res_points(0),
+#endif /*GCALC_DBUG_OFF*/
+  m_res_hook((Gcalc_dyn_list::Item **)&m_result),
+  m_first_active_thread(NULL)
+{}
+
+
 void Gcalc_operation_reducer::init(Gcalc_function *fn, modes mode)
 {
   m_fn= fn;
