@@ -13,7 +13,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "mysqld.h"                             // system_charset_info
 #include "rpl_filter.h"
@@ -105,9 +105,9 @@ Rpl_filter::tables_ok(const char* db, TABLE_LIST* tables)
     if (!tables->updating) 
       continue;
     some_tables_updating= 1;
-    end= strmov(hash_key, tables->db ? tables->db : db);
+    end= strmov(hash_key, tables->db.str ? tables->db.str : db);
     *end++= '.';
-    len= (uint) (strmov(end, tables->table_name) - hash_key);
+    len= (uint) (strmov(end, tables->table_name.str) - hash_key);
     if (do_table_inited) // if there are any do's
     {
       if (my_hash_search(&do_table, (uchar*) hash_key, len))
@@ -239,7 +239,7 @@ Rpl_filter::db_ok_with_wild_table(const char *db)
   int len;
   end= strmov(hash_key, db);
   *end++= '.';
-  len= end - hash_key ;
+  len= (int)(end - hash_key);
   if (wild_do_table_inited && find_wild(&wild_do_table, hash_key, len))
   {
     DBUG_PRINT("return",("1"));
@@ -282,6 +282,9 @@ Rpl_filter::parse_filter_rule(const char* spec, Add_filter add)
   int status= 0;
   char *arg, *ptr, *pstr;
 
+  if (!spec)
+    return false;
+  
   if (! (ptr= my_strdup(spec, MYF(MY_WME))))
     return true;
 
