@@ -14,7 +14,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <my_global.h>
+#include "mariadb.h"
 #include "sql_priv.h"
 #include "unireg.h"
 #include "rpl_rli.h"
@@ -80,7 +80,7 @@ pack_row(TABLE *table, MY_BITMAP const* cols,
   unsigned int null_mask= 1U;
   for ( ; (field= *p_field) ; p_field++)
   {
-    if (bitmap_is_set(cols, p_field - table->field))
+    if (bitmap_is_set(cols, (uint)(p_field - table->field)))
     {
       my_ptrdiff_t offset;
       if (field->is_null(rec_offset))
@@ -105,10 +105,10 @@ pack_row(TABLE *table, MY_BITMAP const* cols,
 #endif
         pack_ptr= field->pack(pack_ptr, field->ptr + offset,
                               field->max_data_length());
-        DBUG_PRINT("debug", ("field: %s; real_type: %d, pack_ptr: 0x%lx;"
-                             " pack_ptr':0x%lx; bytes: %d",
-                             field->field_name, field->real_type(),
-                             (ulong) old_pack_ptr, (ulong) pack_ptr,
+        DBUG_PRINT("debug", ("field: %s; real_type: %d, pack_ptr: %p;"
+                             " pack_ptr':%p; bytes: %d",
+                             field->field_name.str, field->real_type(),
+                             old_pack_ptr,pack_ptr,
                              (int) (pack_ptr - old_pack_ptr)));
         DBUG_DUMP("packed_data", old_pack_ptr, pack_ptr - old_pack_ptr);
       }
@@ -196,6 +196,7 @@ unpack_row(rpl_group_info *rgi,
            uchar const **const current_row_end, ulong *const master_reclength,
            uchar const *const row_end)
 {
+  int error;
   DBUG_ENTER("unpack_row");
   DBUG_ASSERT(row_data);
   DBUG_ASSERT(table);
@@ -254,7 +255,7 @@ unpack_row(rpl_group_info *rgi,
       conv_field ? conv_field : *field_ptr;
     DBUG_PRINT("debug", ("Conversion %srequired for field '%s' (#%ld)",
                          conv_field ? "" : "not ",
-                         (*field_ptr)->field_name,
+                         (*field_ptr)->field_name.str,
                          (long) (field_ptr - begin_ptr)));
     DBUG_ASSERT(f != NULL);
 
@@ -262,7 +263,7 @@ unpack_row(rpl_group_info *rgi,
       No need to bother about columns that does not exist: they have
       gotten default values when being emptied above.
      */
-    if (bitmap_is_set(cols, field_ptr -  begin_ptr))
+    if (bitmap_is_set(cols, (uint)(field_ptr -  begin_ptr)))
     {
       if ((null_mask & 0xFF) == 0)
       {
@@ -305,7 +306,7 @@ unpack_row(rpl_group_info *rgi,
           push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
                               ER_BAD_NULL_ERROR,
                               ER_THD(thd, ER_BAD_NULL_ERROR),
-                              f->field_name);
+                              f->field_name.str);
         }
       }
       else
@@ -322,9 +323,9 @@ unpack_row(rpl_group_info *rgi,
 
         pack_ptr= f->unpack(f->ptr, pack_ptr, row_end, metadata);
 	DBUG_PRINT("debug", ("field: %s; metadata: 0x%x;"
-                             " pack_ptr: 0x%lx; pack_ptr': 0x%lx; bytes: %d",
-                             f->field_name, metadata,
-                             (ulong) old_pack_ptr, (ulong) pack_ptr,
+                             " pack_ptr: %p; pack_ptr': %p; bytes: %d",
+                             f->field_name.str, metadata,
+                             old_pack_ptr, pack_ptr,
                              (int) (pack_ptr - old_pack_ptr)));
         if (!pack_ptr)
         {
@@ -336,18 +337,18 @@ unpack_row(rpl_group_info *rgi,
               Galera Node throws "Could not read field" error and drops out of cluster
             */
             WSREP_WARN("ROW event unpack field: %s  metadata: 0x%x;"
-                       " pack_ptr: 0x%lx; conv_table %p conv_field %p table %s"
-                       " row_end: 0x%lx",
-                       f->field_name, metadata,
-                       (ulong) old_pack_ptr, conv_table, conv_field,
-                       (table_found) ? "found" : "not found", (ulong)row_end
+                       " pack_ptr: %p; conv_table %p conv_field %p table %s"
+                       " row_end: %p",
+                       f->field_name.str, metadata,
+                       old_pack_ptr, conv_table, conv_field,
+                       (table_found) ? "found" : "not found", row_end
             );
 	  }
 
           rgi->rli->report(ERROR_LEVEL, ER_SLAVE_CORRUPT_EVENT,
                       rgi->gtid_info(),
                       "Could not read field '%s' of table '%s.%s'",
-                      f->field_name, table->s->db.str,
+                      f->field_name.str, table->s->db.str,
                       table->s->table_name.str);
           DBUG_RETURN(HA_ERR_CORRUPT_EVENT);
         }
@@ -370,7 +371,7 @@ unpack_row(rpl_group_info *rgi,
         conv_field->sql_type(source_type);
         conv_field->val_str(&value_string);
         DBUG_PRINT("debug", ("Copying field '%s' of type '%s' with value '%s'",
-                             (*field_ptr)->field_name,
+                             (*field_ptr)->field_name.str,
                              source_type.c_ptr_safe(), value_string.c_ptr_safe()));
 #endif
         copy.set(*field_ptr, f, TRUE);
@@ -381,7 +382,7 @@ unpack_row(rpl_group_info *rgi,
         (*field_ptr)->sql_type(target_type);
         (*field_ptr)->val_str(&value_string);
         DBUG_PRINT("debug", ("Value of field '%s' of type '%s' is now '%s'",
-                             (*field_ptr)->field_name,
+                             (*field_ptr)->field_name.str,
                              target_type.c_ptr_safe(), value_string.c_ptr_safe()));
 #endif
       }
@@ -416,6 +417,12 @@ unpack_row(rpl_group_info *rgi,
     }
   }
 
+  /*
+    Add Extra slave persistent columns
+  */
+  if (unlikely(error= fill_extra_persistent_columns(table, cols->n_bits)))
+    DBUG_RETURN(error);
+
   /*
     We should now have read all the null bytes, otherwise something is
     really wrong.
@@ -428,7 +435,7 @@ unpack_row(rpl_group_info *rgi,
   if (master_reclength)
   {
     if (*field_ptr)
-      *master_reclength = (*field_ptr)->ptr - table->record[0];
+      *master_reclength = (ulong)((*field_ptr)->ptr - table->record[0]);
     else
       *master_reclength = table->s->reclength;
   }
@@ -483,11 +490,36 @@ int prepare_record(TABLE *const table, const uint skip, const bool check)
                           Sql_condition::WARN_LEVEL_WARN,
                           ER_NO_DEFAULT_FOR_FIELD,
                           ER_THD(thd, ER_NO_DEFAULT_FOR_FIELD),
-                          f->field_name);
+                          f->field_name.str);
     }
   }
 
   DBUG_RETURN(0);
 }
+/**
+  Fills @c table->record[0] with computed values of extra  persistent  column which are present on slave but not on master.
+  @param table         Table whose record[0] buffer is prepared.
+  @param master_cols   No of columns on master 
+  @returns 0 on        success
+ */
+int fill_extra_persistent_columns(TABLE *table, int master_cols)
+{
+  int error= 0;
+  Field **vfield_ptr, *vfield;
 
+  if (!table->vfield)
+    return 0;
+  for (vfield_ptr= table->vfield; *vfield_ptr; ++vfield_ptr)
+  {
+    vfield= *vfield_ptr;
+    if (vfield->field_index >= master_cols && vfield->stored_in_db())
+    {
+      /*Set bitmap for writing*/
+      bitmap_set_bit(table->vcol_set, vfield->field_index);
+      error= vfield->vcol_info->expr->save_in_field(vfield,0);
+      bitmap_clear_bit(table->vcol_set, vfield->field_index);
+    }
+  }
+  return error;
+}
 #endif // HAVE_REPLICATION
