@@ -1,5 +1,5 @@
 /* Copyright (c) 2010, 2015, Oracle and/or its affiliates.
-   Copyright (c) 2011, 2016, MariaDB
+   Copyright (c) 2011, 2019, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,7 +14,8 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include "sql_class.h"                       // THD and my_global.h
+#include "mariadb.h"
+#include "sql_class.h"                       // THD
 #include "keycaches.h"                       // get_key_cache
 #include "sql_base.h"                        // Open_table_context
 #include "lock.h"                            // MYSQL_OPEN_*
@@ -54,7 +55,7 @@ static bool admin_recreate_table(THD *thd, TABLE_LIST *table_list)
 
   DEBUG_SYNC(thd, "ha_admin_try_alter");
   tmp_disable_binlog(thd); // binlogging is done by caller if wanted
-  result_code= (open_temporary_tables(thd, table_list) ||
+  result_code= (thd->open_temporary_tables(table_list) ||
                 mysql_recreate_table(thd, table_list, false));
   reenable_binlog(thd);
   /*
@@ -66,7 +67,6 @@ static bool admin_recreate_table(THD *thd, TABLE_LIST *table_list)
   if (thd->get_stmt_da()->is_ok())
     thd->get_stmt_da()->reset_diagnostics_area();
   table_list->table= NULL;
-  result_code= result_code ? HA_ADMIN_FAILED : HA_ADMIN_OK;
   DBUG_RETURN(result_code);
 }
 
@@ -77,7 +77,7 @@ static int send_check_errmsg(THD *thd, TABLE_LIST* table,
 {
   Protocol *protocol= thd->protocol;
   protocol->prepare_for_resend();
-  protocol->store(table->alias, system_charset_info);
+  protocol->store(table->alias.str, table->alias.length, system_charset_info);
   protocol->store((char*) operator_name, system_charset_info);
   protocol->store(STRING_WITH_LEN("error"), system_charset_info);
   protocol->store(errmsg, system_charset_info);
@@ -123,7 +123,7 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
     */
 
     table_list->mdl_request.init(MDL_key::TABLE,
-                                 table_list->db, table_list->table_name,
+                                 table_list->db.str, table_list->table_name.str,
                                  MDL_EXCLUSIVE, MDL_TRANSACTION);
 
     if (lock_table_names(thd, table_list, table_list->next_global,
@@ -131,11 +131,12 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
       DBUG_RETURN(0);
     has_mdl_lock= TRUE;
 
-    share= tdc_acquire_share_shortlived(thd, table_list, GTS_TABLE);
+    share= tdc_acquire_share(thd, table_list, GTS_TABLE);
     if (share == NULL)
       DBUG_RETURN(0);				// Can't open frm file
 
-    if (open_table_from_share(thd, share, "", 0, 0, 0, &tmp_table, FALSE))
+    if (open_table_from_share(thd, share, &empty_clex_str, 0, 0, 0,
+                              &tmp_table, FALSE))
     {
       tdc_release_share(share);
       DBUG_RETURN(0);                           // Out of memory
@@ -163,7 +164,7 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
     - Run a normal repair using the new index file and the old data file
   */
 
-  if (table->s->frm_version != FRM_VER_TRUE_VARCHAR &&
+  if (table->s->frm_version < FRM_VER_TRUE_VARCHAR &&
       table->s->varchar_fields)
   {
     error= send_check_errmsg(thd, table_list, "repair",
@@ -189,7 +190,7 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
   if (!mysql_file_stat(key_file_misc, from, &stat_info, MYF(0)))
     goto end;				// Can't use USE_FRM flag
 
-  my_snprintf(tmp, sizeof(tmp), "%s-%lx_%lx",
+  my_snprintf(tmp, sizeof(tmp), "%s-%lx_%llx",
 	      from, current_pid, thd->thread_id);
 
   if (table_list->table)
@@ -218,7 +219,7 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
 			     "Failed renaming data file");
     goto end;
   }
-  if (dd_recreate_table(thd, table_list->db, table_list->table_name))
+  if (dd_recreate_table(thd, table_list->db.str, table_list->table_name.str))
   {
     error= send_check_errmsg(thd, table_list, "repair",
 			     "Failed generating table from .frm file");
@@ -239,7 +240,7 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
 
   if (thd->locked_tables_list.locked_tables())
   {
-    if (thd->locked_tables_list.reopen_tables(thd))
+    if (thd->locked_tables_list.reopen_tables(thd, false))
       goto end;
     /* Restore the table in the table list with the new opened table */
     table_list->table= pos_in_locked_tables->table;
@@ -261,9 +262,12 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
 end:
   thd->locked_tables_list.unlink_all_closed_tables(thd, NULL, 0);
   if (table == &tmp_table)
-    closefrm(table, 1);				// Free allocated memory
+  {
+    closefrm(table);
+    tdc_release_share(table->s);
+  }
   /* In case of a temporary table there will be no metadata lock. */
-  if (error && has_mdl_lock)
+  if (unlikely(error) && has_mdl_lock)
     thd->mdl_context.release_transactional_locks();
 
   DBUG_RETURN(error);
@@ -292,6 +296,129 @@ static inline bool table_not_corrupt_error(uint sql_errno)
           sql_errno == ER_WRONG_OBJECT);
 }
 
+#ifndef DBUG_OFF
+// It is counter for debugging fail on second call of open_only_one_table
+static int debug_fail_counter= 0;
+#endif
+
+static bool open_only_one_table(THD* thd, TABLE_LIST* table,
+                                bool repair_table_use_frm,
+                                bool is_view_operator_func)
+{
+  LEX *lex= thd->lex;
+  SELECT_LEX *select= &lex->select_lex;
+  TABLE_LIST *save_next_global, *save_next_local;
+  bool open_error;
+  save_next_global= table->next_global;
+  table->next_global= 0;
+  save_next_local= table->next_local;
+  table->next_local= 0;
+  select->table_list.first= table;
+  /*
+    Time zone tables and SP tables can be add to lex->query_tables list,
+    so it have to be prepared.
+    TODO: Investigate if we can put extra tables into argument instead of
+    using lex->query_tables
+  */
+  lex->query_tables= table;
+  lex->query_tables_last= &table->next_global;
+  lex->query_tables_own_last= 0;
+
+  DBUG_EXECUTE_IF("fail_2call_open_only_one_table", {
+                  if (debug_fail_counter)
+                  {
+                    open_error= TRUE;
+                    goto dbug_err;
+                  }
+                  else
+                    debug_fail_counter++;
+                  });
+
+  /*
+    CHECK TABLE command is allowed for views as well. Check on alter flags
+    to differentiate from ALTER TABLE...CHECK PARTITION on which view is not
+    allowed.
+  */
+  if (lex->alter_info.partition_flags & ALTER_PARTITION_ADMIN ||
+      !is_view_operator_func)
+  {
+    table->required_type= TABLE_TYPE_NORMAL;
+    DBUG_ASSERT(lex->table_type != TABLE_TYPE_VIEW);
+  }
+  else if (lex->table_type == TABLE_TYPE_VIEW)
+  {
+    table->required_type= lex->table_type;
+  }
+  else if ((lex->table_type != TABLE_TYPE_VIEW) &&
+           lex->sql_command == SQLCOM_REPAIR)
+  {
+    table->required_type= TABLE_TYPE_NORMAL;
+  }
+
+  if (lex->sql_command == SQLCOM_CHECK ||
+      lex->sql_command == SQLCOM_REPAIR ||
+      lex->sql_command == SQLCOM_ANALYZE ||
+      lex->sql_command == SQLCOM_OPTIMIZE)
+    thd->prepare_derived_at_open= TRUE;
+  if (!thd->locked_tables_mode && repair_table_use_frm)
+  {
+    /*
+      If we're not under LOCK TABLES and we're executing REPAIR TABLE
+      USE_FRM, we need to ignore errors from open_and_lock_tables().
+      REPAIR TABLE USE_FRM is a heavy weapon used when a table is
+      critically damaged, so open_and_lock_tables() will most likely
+      report errors. Those errors are not interesting for the user
+      because it's already known that the table is badly damaged.
+    */
+
+    Diagnostics_area *da= thd->get_stmt_da();
+    Warning_info tmp_wi(thd->query_id, false, true);
+
+    da->push_warning_info(&tmp_wi);
+
+    open_error= (thd->open_temporary_tables(table) ||
+                 open_and_lock_tables(thd, table, TRUE, 0));
+
+    da->pop_warning_info();
+  }
+  else
+  {
+    /*
+      It's assumed that even if it is REPAIR TABLE USE_FRM, the table
+      can be opened if we're under LOCK TABLES (otherwise LOCK TABLES
+      would fail). Thus, the only errors we could have from
+      open_and_lock_tables() are logical ones, like incorrect locking
+      mode. It does make sense for the user to see such errors.
+    */
+
+    open_error= (thd->open_temporary_tables(table) ||
+                 open_and_lock_tables(thd, table, TRUE, 0));
+  }
+
+#ifndef DBUG_OFF
+dbug_err:
+#endif
+
+  thd->prepare_derived_at_open= FALSE;
+
+  /*
+    MERGE engine may adjust table->next_global chain, thus we have to
+    append save_next_global after merge children.
+  */
+  if (save_next_global)
+  {
+    TABLE_LIST *table_list_iterator= table;
+    while (table_list_iterator->next_global)
+      table_list_iterator= table_list_iterator->next_global;
+    table_list_iterator->next_global= save_next_global;
+    save_next_global->prev_global= &table_list_iterator->next_global;
+  }
+
+  table->next_local= save_next_local;
+
+  return open_error;
+}
+
 
 /*
   RETURN VALUES
@@ -303,7 +430,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
                               HA_CHECK_OPT* check_opt,
                               const char *operator_name,
                               thr_lock_type lock_type,
-                              bool open_for_modify,
+                              bool org_open_for_modify,
                               bool repair_table_use_frm,
                               uint extra_open_options,
                               int (*prepare_func)(THD *, TABLE_LIST *,
@@ -314,7 +441,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
                                                        HA_CHECK_OPT *))
 {
   TABLE_LIST *table;
-  SELECT_LEX *select= &thd->lex->select_lex;
   List<Item> field_list;
   Item *item;
   Protocol *protocol= thd->protocol;
@@ -323,7 +449,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
   int compl_result_code;
   bool need_repair_or_alter= 0;
   wait_for_commit* suspended_wfc;
-
   DBUG_ENTER("mysql_admin_table");
   DBUG_PRINT("enter", ("extra_open_options: %u", extra_open_options));
 
@@ -366,13 +491,14 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
   for (table= tables; table; table= table->next_local)
   {
     char table_name[SAFE_NAME_LEN*2+2];
-    char* db = table->db;
+    const char *db= table->db.str;
     bool fatal_error=0;
     bool open_error;
     bool collect_eis=  FALSE;
+    bool open_for_modify= org_open_for_modify;
 
-    DBUG_PRINT("admin", ("table: '%s'.'%s'", table->db, table->table_name));
-    strxmov(table_name, db, ".", table->table_name, NullS);
+    DBUG_PRINT("admin", ("table: '%s'.'%s'", db, table->table_name.str));
+    strxmov(table_name, db, ".", table->table_name.str, NullS);
     thd->open_options|= extra_open_options;
     table->lock_type= lock_type;
     /*
@@ -389,104 +515,16 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
     /* open only one table from local list of command */
     while (1)
     {
-      TABLE_LIST *save_next_global, *save_next_local;
-      save_next_global= table->next_global;
-      table->next_global= 0;
-      save_next_local= table->next_local;
-      table->next_local= 0;
-      select->table_list.first= table;
-      /*
-        Time zone tables and SP tables can be add to lex->query_tables list,
-        so it have to be prepared.
-        TODO: Investigate if we can put extra tables into argument instead of
-        using lex->query_tables
-      */
-      lex->query_tables= table;
-      lex->query_tables_last= &table->next_global;
-      lex->query_tables_own_last= 0;
-
-      /*
-        CHECK TABLE command is allowed for views as well. Check on alter flags
-        to differentiate from ALTER TABLE...CHECK PARTITION on which view is not
-        allowed.
-      */
-      if (lex->alter_info.flags & Alter_info::ALTER_ADMIN_PARTITION ||
-          view_operator_func == NULL)
-      {
-        table->required_type=FRMTYPE_TABLE;
-        DBUG_ASSERT(!lex->only_view);
-      }
-      else if (lex->only_view)
-      {
-        table->required_type= FRMTYPE_VIEW;
-      }
-      else if (!lex->only_view && lex->sql_command == SQLCOM_REPAIR)
-      {
-        table->required_type= FRMTYPE_TABLE;
-      }
-
-      if (lex->sql_command == SQLCOM_CHECK ||
-          lex->sql_command == SQLCOM_REPAIR ||
-          lex->sql_command == SQLCOM_ANALYZE ||
-          lex->sql_command == SQLCOM_OPTIMIZE)
-	thd->prepare_derived_at_open= TRUE;
-      if (!thd->locked_tables_mode && repair_table_use_frm)
-      {
-        /*
-          If we're not under LOCK TABLES and we're executing REPAIR TABLE
-          USE_FRM, we need to ignore errors from open_and_lock_tables().
-          REPAIR TABLE USE_FRM is a heavy weapon used when a table is
-          critically damaged, so open_and_lock_tables() will most likely
-          report errors. Those errors are not interesting for the user
-          because it's already known that the table is badly damaged.
-        */
-
-        Diagnostics_area *da= thd->get_stmt_da();
-        Warning_info tmp_wi(thd->query_id, false, true);
-
-        da->push_warning_info(&tmp_wi);
-
-        open_error= (open_temporary_tables(thd, table) ||
-                     open_and_lock_tables(thd, table, TRUE, 0));
-
-        da->pop_warning_info();
-      }
-      else
-      {
-        /*
-          It's assumed that even if it is REPAIR TABLE USE_FRM, the table
-          can be opened if we're under LOCK TABLES (otherwise LOCK TABLES
-          would fail). Thus, the only errors we could have from
-          open_and_lock_tables() are logical ones, like incorrect locking
-          mode. It does make sense for the user to see such errors.
-        */
-
-        open_error= (open_temporary_tables(thd, table) ||
-                     open_and_lock_tables(thd, table, TRUE, 0));
-      }
-      thd->prepare_derived_at_open= FALSE;
-
-      /*
-        MERGE engine may adjust table->next_global chain, thus we have to
-        append save_next_global after merge children.
-      */
-      if (save_next_global)
-      {
-        TABLE_LIST *table_list_iterator= table;
-        while (table_list_iterator->next_global)
-          table_list_iterator= table_list_iterator->next_global;
-        table_list_iterator->next_global= save_next_global;
-        save_next_global->prev_global= &table_list_iterator->next_global;
-      }
-
-      table->next_local= save_next_local;
+      open_error= open_only_one_table(thd, table,
+                                      repair_table_use_frm,
+                                      (view_operator_func != NULL));
       thd->open_options&= ~extra_open_options;
 
       /*
         If open_and_lock_tables() failed, close_thread_tables() will close
         the table and table->table can therefore be invalid.
       */
-      if (open_error)
+      if (unlikely(open_error))
         table->table= NULL;
 
       /*
@@ -494,7 +532,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
         so any errors opening the table are logical errors.
         In these cases it does not make sense to try to repair.
       */
-      if (open_error && thd->locked_tables_mode)
+      if (unlikely(open_error) && thd->locked_tables_mode)
       {
         result_code= HA_ADMIN_FAILED;
         goto send_result;
@@ -509,7 +547,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
       close_thread_tables(thd);
       table->table= NULL;
       thd->mdl_context.release_transactional_locks();
-      table->mdl_request.init(MDL_key::TABLE, table->db, table->table_name,
+      table->mdl_request.init(MDL_key::TABLE, table->db.str, table->table_name.str,
                               MDL_SHARED_NO_READ_WRITE, MDL_TRANSACTION);
     }
 
@@ -523,12 +561,13 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
         */
         Alter_info *alter_info= &lex->alter_info;
 
-        if (alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION)
+        if (alter_info->partition_flags & ALTER_PARTITION_ADMIN)
         {
           if (!table->table->part_info)
           {
             my_error(ER_PARTITION_MGMT_ON_NONPARTITIONED, MYF(0));
-            goto err2;
+            thd->resume_subsequent_commits(suspended_wfc);
+            DBUG_RETURN(TRUE);
           }
           if (set_part_state(alter_info, table->table->part_info, PART_ADMIN))
           {
@@ -551,7 +590,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
         }
       }
 #endif
-    DBUG_PRINT("admin", ("table: 0x%lx", (long) table->table));
+    DBUG_PRINT("admin", ("table: %p", table->table));
 
     if (prepare_func)
     {
@@ -664,8 +703,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
     if (lock_type == TL_WRITE && !table->table->s->tmp_table &&
                         table->mdl_request.type > MDL_SHARED_WRITE)
     {
-      if (wait_while_table_is_used(thd, table->table,
-                                   HA_EXTRA_PREPARE_FOR_RENAME))
+      if (wait_while_table_is_used(thd, table->table, HA_EXTRA_NOT_USED))
         goto err;
       DEBUG_SYNC(thd, "after_admin_flush");
       /* Flush entries in the query cache involving this table. */
@@ -720,7 +758,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
     if (operator_func == &handler::ha_analyze)
     {
       TABLE *tab= table->table;
-      Field **field_ptr= tab->field;
 
       if (lex->with_persistent_for_clause &&
           tab->s->table_category != TABLE_CATEGORY_USER)
@@ -732,58 +769,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
          (get_use_stat_tables_mode(thd) > NEVER ||
           lex->with_persistent_for_clause));
 
-      if (collect_eis)
-      {
-        if (!lex->column_list)
-        {
-          bitmap_clear_all(tab->read_set);
-          for (uint fields= 0; *field_ptr; field_ptr++, fields++)
-          {
-            enum enum_field_types type= (*field_ptr)->type();
-            if (type < MYSQL_TYPE_MEDIUM_BLOB ||
-                type > MYSQL_TYPE_BLOB)
-              bitmap_set_bit(tab->read_set, fields);
-            else if (collect_eis)
-              push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                                  ER_NO_EIS_FOR_FIELD,
-                                  ER_THD(thd, ER_NO_EIS_FOR_FIELD),
-                                  (*field_ptr)->field_name);
-          }
-        }
-        else
-        {
-          int pos;
-          LEX_STRING *column_name;
-          List_iterator_fast<LEX_STRING> it(*lex->column_list);
-
-          bitmap_clear_all(tab->read_set);
-          while ((column_name= it++))
-          {
-            if (tab->s->fieldnames.type_names == 0 ||
-                (pos= find_type(&tab->s->fieldnames, column_name->str,
-                                column_name->length, 1)) <= 0)
-            {
-              compl_result_code= result_code= HA_ADMIN_INVALID;
-              break;
-            }
-            pos--;
-            enum enum_field_types type= tab->field[pos]->type();
-            if (type < MYSQL_TYPE_MEDIUM_BLOB ||
-                type > MYSQL_TYPE_BLOB)
-              bitmap_set_bit(tab->read_set, pos);
-            else if (collect_eis)
-              push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                                  ER_NO_EIS_FOR_FIELD,
-                                  ER_THD(thd, ER_NO_EIS_FOR_FIELD),
-                                  column_name->str);
-          }
-          tab->file->column_bitmaps_signal(); 
-        }
-      }
-      else
-      {
-        DBUG_ASSERT(!lex->column_list);
-      }
 
       if (!lex->index_list)
       {
@@ -821,11 +806,86 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
 
     if (compl_result_code == HA_ADMIN_OK && collect_eis)
     {
-      if (!(compl_result_code=
-            alloc_statistics_for_table(thd, table->table)) &&
-          !(compl_result_code=
-            collect_statistics_for_table(thd, table->table)))
-        compl_result_code= update_statistics_for_table(thd, table->table);
+      /*
+        Here we close and reopen table in read mode because operation of
+        collecting statistics is long and it will be better do not block
+        the table completely.
+        InnoDB will allow read/write and MyISAM read/insert.
+      */
+      trans_commit_stmt(thd);
+      trans_commit(thd);
+      thd->open_options|= extra_open_options;
+      close_thread_tables(thd);
+      table->table= NULL;
+      thd->mdl_context.release_transactional_locks();
+      table->mdl_request.init(MDL_key::TABLE, table->db.str, table->table_name.str,
+                              MDL_SHARED_NO_READ_WRITE, MDL_TRANSACTION);
+      table->mdl_request.set_type(MDL_SHARED_READ);
+
+      table->lock_type= TL_READ;
+      DBUG_ASSERT(view_operator_func == NULL);
+      open_error= open_only_one_table(thd, table,
+                                      repair_table_use_frm, FALSE);
+      thd->open_options&= ~extra_open_options;
+
+      if (unlikely(!open_error))
+      {
+        TABLE *tab= table->table;
+        Field **field_ptr= tab->field;
+        if (!lex->column_list)
+        {
+          bitmap_clear_all(tab->read_set);
+          for (uint fields= 0; *field_ptr; field_ptr++, fields++)
+          {
+            enum enum_field_types type= (*field_ptr)->type();
+            if (type < MYSQL_TYPE_MEDIUM_BLOB ||
+                type > MYSQL_TYPE_BLOB)
+              bitmap_set_bit(tab->read_set, fields);
+            else
+              push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                                  ER_NO_EIS_FOR_FIELD,
+                                  ER_THD(thd, ER_NO_EIS_FOR_FIELD),
+                                  (*field_ptr)->field_name.str);
+          }
+        }
+        else
+        {
+          int pos;
+          LEX_STRING *column_name;
+          List_iterator_fast<LEX_STRING> it(*lex->column_list);
+
+          bitmap_clear_all(tab->read_set);
+          while ((column_name= it++))
+          {
+            if (tab->s->fieldnames.type_names == 0 ||
+                (pos= find_type(&tab->s->fieldnames, column_name->str,
+                                column_name->length, 1)) <= 0)
+            {
+              compl_result_code= result_code= HA_ADMIN_INVALID;
+              break;
+            }
+            pos--;
+            enum enum_field_types type= tab->field[pos]->type();
+            if (type < MYSQL_TYPE_MEDIUM_BLOB ||
+                type > MYSQL_TYPE_BLOB)
+              bitmap_set_bit(tab->read_set, pos);
+            else
+              push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
+                                  ER_NO_EIS_FOR_FIELD,
+                                  ER_THD(thd, ER_NO_EIS_FOR_FIELD),
+                                  column_name->str);
+          }
+          tab->file->column_bitmaps_signal();
+        }
+        if (!(compl_result_code=
+              alloc_statistics_for_table(thd, table->table)) &&
+            !(compl_result_code=
+              collect_statistics_for_table(thd, table->table)))
+          compl_result_code= update_statistics_for_table(thd, table->table);
+      }
+      else
+        compl_result_code= HA_ADMIN_FAILED;
+
       if (compl_result_code)
         result_code= HA_ADMIN_FAILED;
       else
@@ -942,7 +1002,7 @@ send_result_message:
       Alter_info *alter_info= &lex->alter_info;
 
       protocol->store(STRING_WITH_LEN("note"), system_charset_info);
-      if (alter_info->flags & Alter_info::ALTER_ADMIN_PARTITION)
+      if (alter_info->partition_flags & ALTER_PARTITION_ADMIN)
       {
         protocol->store(STRING_WITH_LEN(
         "Table does not support optimize on partitions. All partitions "
@@ -978,18 +1038,18 @@ send_result_message:
         table->mdl_request.ticket= NULL;
         DEBUG_SYNC(thd, "ha_admin_open_ltable");
         table->mdl_request.set_type(MDL_SHARED_WRITE);
-        if (!open_temporary_tables(thd, table) &&
+        if (!thd->open_temporary_tables(table) &&
             (table->table= open_ltable(thd, table, lock_type, 0)))
         {
-          uint save_flags;
+          ulonglong save_flags;
           /* Store the original value of alter_info->flags */
           save_flags= alter_info->flags;
 
           /*
-           Reset the ALTER_ADMIN_PARTITION bit in alter_info->flags
+           Reset the ALTER_PARTITION_ADMIN bit in alter_info->flags
            to force analyze on all partitions.
           */
-          alter_info->flags &= ~(Alter_info::ALTER_ADMIN_PARTITION);
+          alter_info->partition_flags &= ~(ALTER_PARTITION_ADMIN);
           result_code= table->table->file->ha_analyze(thd, check_opt);
           if (result_code == HA_ADMIN_ALREADY_DONE)
             result_code= HA_ADMIN_OK;
@@ -1058,11 +1118,11 @@ send_result_message:
       if (what_to_upgrade)
         length= my_snprintf(buf, sizeof(buf),
                             ER_THD(thd, ER_TABLE_NEEDS_UPGRADE),
-                            what_to_upgrade, table->table_name);
+                            what_to_upgrade, table->table_name.str);
       else
         length= my_snprintf(buf, sizeof(buf),
                             ER_THD(thd, ER_TABLE_NEEDS_REBUILD),
-                            table->table_name);
+                            table->table_name.str);
       protocol->store(buf, length, system_charset_info);
       fatal_error=1;
       break;
@@ -1094,7 +1154,7 @@ send_result_message:
       else if (open_for_modify || fatal_error)
       {
         tdc_remove_table(thd, TDC_RT_REMOVE_UNUSED,
-                         table->db, table->table_name, FALSE);
+                         table->db.str, table->table_name.str, FALSE);
         /*
           May be something modified. Consequently, we have to
           invalidate the query cache.
@@ -1104,7 +1164,7 @@ send_result_message:
       }
     }
     /* Error path, a admin command failed. */
-    if (thd->transaction_rollback_request)
+    if (thd->transaction_rollback_request || fatal_error)
     {
       /*
         Unlikely, but transaction rollback was requested by one of storage
@@ -1115,7 +1175,9 @@ send_result_message:
     }
     else
     {
-      if (trans_commit_stmt(thd) || trans_commit_implicit(thd))
+      if (trans_commit_stmt(thd) ||
+          (stmt_causes_implicit_commit(thd, CF_IMPLICIT_COMMIT_END) &&
+           trans_commit_implicit(thd)))
         goto err;
     }
     close_thread_tables(thd);
@@ -1149,7 +1211,8 @@ send_result_message:
 err:
   /* Make sure this table instance is not reused after the failure. */
   trans_rollback_stmt(thd);
-  trans_rollback(thd);
+  if (stmt_causes_implicit_commit(thd, CF_IMPLICIT_COMMIT_END))
+    trans_rollback(thd);
   if (table && table->table)
   {
     table->table->m_needs_reopen= true;
@@ -1157,7 +1220,6 @@ err:
   }
   close_thread_tables(thd);			// Shouldn't be needed
   thd->mdl_context.release_transactional_locks();
-err2:
   thd->resume_subsequent_commits(suspended_wfc);
   DBUG_RETURN(TRUE);
 }
@@ -1177,7 +1239,7 @@ err2:
 */
 
 bool mysql_assign_to_keycache(THD* thd, TABLE_LIST* tables,
-			     LEX_STRING *key_cache_name)
+			     const LEX_CSTRING *key_cache_name)
 {
   HA_CHECK_OPT check_opt;
   KEY_CACHE *key_cache;
@@ -1244,7 +1306,6 @@ bool Sql_cmd_analyze_table::execute(THD *thd)
                          FALSE, UINT_MAX, FALSE))
     goto error;
   WSREP_TO_ISOLATION_BEGIN_WRTCHK(NULL, NULL, first_table);
-  thd->enable_slow_log= opt_log_slow_admin_statements;
   res= mysql_admin_table(thd, first_table, &m_lex->check_opt,
                          "analyze", lock_type, 1, 0, 0, 0,
                          &handler::ha_analyze, 0);
@@ -1260,6 +1321,7 @@ bool Sql_cmd_analyze_table::execute(THD *thd)
   m_lex->query_tables= first_table;
 
 error:
+WSREP_ERROR_LABEL:
   DBUG_RETURN(res);
 }
 
@@ -1275,7 +1337,6 @@ bool Sql_cmd_check_table::execute(THD *thd)
   if (check_table_access(thd, SELECT_ACL, first_table,
                          TRUE, UINT_MAX, FALSE))
     goto error; /* purecov: inspected */
-  thd->enable_slow_log= opt_log_slow_admin_statements;
 
   res= mysql_admin_table(thd, first_table, &m_lex->check_opt, "check",
                          lock_type, 0, 0, HA_OPEN_FOR_REPAIR, 0,
@@ -1300,7 +1361,7 @@ bool Sql_cmd_optimize_table::execute(THD *thd)
                          FALSE, UINT_MAX, FALSE))
     goto error; /* purecov: inspected */
   WSREP_TO_ISOLATION_BEGIN_WRTCHK(NULL, NULL, first_table);
-  thd->enable_slow_log= opt_log_slow_admin_statements;
+
   res= (specialflag & SPECIAL_NO_NEW_FUNC) ?
     mysql_recreate_table(thd, first_table, true) :
     mysql_admin_table(thd, first_table, &m_lex->check_opt,
@@ -1318,6 +1379,7 @@ bool Sql_cmd_optimize_table::execute(THD *thd)
   m_lex->query_tables= first_table;
 
 error:
+WSREP_ERROR_LABEL:
   DBUG_RETURN(res);
 }
 
@@ -1332,7 +1394,6 @@ bool Sql_cmd_repair_table::execute(THD *thd)
   if (check_table_access(thd, SELECT_ACL | INSERT_ACL, first_table,
                          FALSE, UINT_MAX, FALSE))
     goto error; /* purecov: inspected */
-  thd->enable_slow_log= opt_log_slow_admin_statements;
   WSREP_TO_ISOLATION_BEGIN_WRTCHK(NULL, NULL, first_table);
   res= mysql_admin_table(thd, first_table, &m_lex->check_opt, "repair",
                          TL_WRITE, 1,
@@ -1352,5 +1413,6 @@ bool Sql_cmd_repair_table::execute(THD *thd)
   m_lex->query_tables= first_table;
 
 error:
+WSREP_ERROR_LABEL:
   DBUG_RETURN(res);
 }
