@@ -32,7 +32,9 @@
 /****************************************************************************/
 #include "mycat.h"
 
-static char *strz(PGLOBAL g, LEX_STRING &ls);
+#if defined(JAVA_SUPPORT) || defined(CMGO_SUPPORT)
+bool MongoEnabled(void);
+#endif   // JAVA_SUPPORT || CMGO_SUPPORT
 
 /****************************************************************************/
 /*  Structures used to pass info between CONNECT and ha_connect.            */
@@ -166,8 +168,8 @@ public:
   ~ha_connect();
 
   // CONNECT Implementation
-  static   bool connect_init(void);
-  static   bool connect_end(void);
+//static   bool connect_init(void);
+//static   bool connect_end(void);
   TABTYPE  GetRealType(PTOS pos= NULL);
   char    *GetRealString(PCSZ s);
 	PCSZ     GetStringOption(PCSZ opname, PCSZ sdef= NULL);
@@ -207,13 +209,13 @@ public:
   bool     IsOpened(void);
   int      CloseTable(PGLOBAL g);
   int      MakeRecord(char *buf);
-  int      ScanRecord(PGLOBAL g, uchar *buf);
-  int      CheckRecord(PGLOBAL g, const uchar *oldbuf, uchar *newbuf);
+  int      ScanRecord(PGLOBAL g, const uchar *buf);
+  int      CheckRecord(PGLOBAL g, const uchar *oldbuf, const uchar *newbuf);
 	int      ReadIndexed(uchar *buf, OPVAL op, const key_range *kr= NULL);
 	bool     IsIndexed(Field *fp);
   bool     MakeKeyWhere(PGLOBAL g, PSTRG qry, OPVAL op, char q,
                                    const key_range *kr);
-  inline char *Strz(LEX_STRING &ls);
+//inline char *Strz(LEX_STRING &ls);
 	key_range start_key;
 
 
@@ -231,7 +233,7 @@ public:
   /** @brief
     The file extensions.
    */
-  const char **bas_ext() const;
+//const char **bas_ext() const;
 
  /**
     Check if a storage engine supports a particular alter table in-place
@@ -348,6 +350,9 @@ const char *GetValStr(OPVAL vop, bool neg);
 PFIL  CondFilter(PGLOBAL g, Item *cond);
 //PFIL  CheckFilter(PGLOBAL g);
 
+/** admin commands - called from mysql_admin_table */
+virtual int check(THD* thd, HA_CHECK_OPT* check_opt);
+
  /**
    Number of rows in table. It will only be called if
    (table_flags() & (HA_HAS_RECORDS | HA_STATS_RECORDS_IS_EXACT)) != 0
@@ -389,7 +394,7 @@ PFIL  CondFilter(PGLOBAL g, Item *cond);
     We implement this in ha_connect.cc. It's not an obligatory method;
     skip it and and MySQL will treat it as not implemented.
   */
-  int update_row(const uchar *old_data, uchar *new_data);
+  int update_row(const uchar *old_data, const uchar *new_data);
 
   /** @brief
     We implement this in ha_connect.cc. It's not an obligatory method;
@@ -504,7 +509,7 @@ private:
   DsMrr_impl ds_mrr;
 
 protected:
-  bool check_privileges(THD *thd, PTOS options, char *dbn, bool quick=false);
+  bool check_privileges(THD *thd, PTOS options, const char *dbn, bool quick=false);
   MODE CheckMode(PGLOBAL g, THD *thd, MODE newmode, bool *chk, bool *cras);
   char *GetDBfromName(const char *name);
 
