@@ -314,12 +314,15 @@ int maria_extra(MARIA_HA *info, enum ha_extra_function function,
     share->state.open_count= 1;
     share->changed= 1;
     _ma_mark_file_changed_now(share);
-    /* Fall through */
+    if (share->temporary)
+      break;
+    /* fall through */
   case HA_EXTRA_PREPARE_FOR_RENAME:
   {
     my_bool do_flush= MY_TEST(function != HA_EXTRA_PREPARE_FOR_DROP);
     my_bool save_global_changed;
     enum flush_type type;
+    DBUG_ASSERT(!share->temporary);
     /*
       This share, to have last_version=0, needs to save all its data/index
       blocks to disk if this is not for a DROP TABLE. Otherwise they would be
@@ -345,7 +348,7 @@ int maria_extra(MARIA_HA *info, enum ha_extra_function function,
       _ma_decrement_open_count(info, 0);
     if (info->trn)
     {
-      _ma_remove_table_from_trnman(share, info->trn);
+      _ma_remove_table_from_trnman(info);
       /* Ensure we don't point to the deleted data in trn */
       info->state= info->state_start= &share->state.state;
     }
@@ -408,7 +411,7 @@ int maria_extra(MARIA_HA *info, enum ha_extra_function function,
     if (info->trn)
     {
       mysql_mutex_lock(&share->intern_lock);
-      _ma_remove_table_from_trnman(share, info->trn);
+      _ma_remove_table_from_trnman(info);
       /* Ensure we don't point to the deleted data in trn */
       info->state= info->state_start= &share->state.state;
       mysql_mutex_unlock(&share->intern_lock);    
@@ -660,4 +663,3 @@ my_bool ma_killed_standalone(MARIA_HA *info __attribute__((unused)))
 {
   return 0;
 }
-
