@@ -701,7 +701,7 @@ static char *create_tmpdir(const char *progname)
 {
   static char test_dirname[FN_REFLEN];
   char tmp_name[FN_REFLEN];
-  uint length;
+  size_t length;
 
   /* Create a temporary directory of name TMP-'executable', but without the -t extension */
   fn_format(tmp_name, progname, "", "", MY_REPLACE_DIR | MY_REPLACE_EXT);
@@ -724,7 +724,8 @@ int main(int argc __attribute__((unused)),
 {
   pthread_t tid;
   pthread_attr_t thr_attr;
-  int *param, error, pagen;
+  int *param, error;
+  size_t pagen;
   File tmp_file;
   MY_INIT(argv[0]);
 
@@ -809,7 +810,7 @@ int main(int argc __attribute__((unused)),
             errno);
     exit(1);
   }
-  DBUG_PRINT("info", ("Page cache %d pages", pagen));
+  DBUG_PRINT("info", ("Page cache %zd pages", pagen));
 
   pthread_mutex_lock(&LOCK_thread_count);
   param=(int*) malloc(sizeof(int));
