@@ -1,4 +1,4 @@
-/* Copyright (C) 2009-2015 Kentoku Shiba
+/* Copyright (C) 2009-2018 Kentoku Shiba
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -11,10 +11,12 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #define MYSQL_SERVER 1
+#include <my_global.h>
 #include "mysql_version.h"
+#include "spd_environ.h"
 #if MYSQL_VERSION_ID < 50500
 #include "mysql_priv.h"
 #include <mysql/plugin.h>
@@ -25,6 +27,7 @@
 #include "sql_partition.h"
 #include "sql_base.h"
 #include "sql_servers.h"
+#include "tztime.h"
 #endif
 #include "spd_err.h"
 #include "spd_param.h"
@@ -57,7 +60,14 @@ extern PSI_cond_key spd_key_cond_bg_direct_sql;
 #endif
 
 extern HASH spider_open_connections;
+extern HASH spider_ipport_conns;
 extern pthread_mutex_t spider_conn_mutex;
+extern pthread_mutex_t spider_conn_id_mutex;
+extern pthread_mutex_t spider_ipport_conn_mutex;
+extern ulonglong spider_conn_id;
+
+/* UTC time zone for timestamp columns */
+extern Time_zone *UTC;
 
 uint spider_udf_calc_hash(
   char *key,
@@ -126,7 +136,7 @@ int spider_udf_direct_sql_create_table_list(
       &direct_sql->tables, sizeof(TABLE*) * table_count,
       &tmp_name_ptr, sizeof(char) * (
         table_name_list_length +
-        thd->db_length * table_count +
+        SPIDER_THD_db_length(thd) * table_count +
         2 * table_count
       ),
       &direct_sql->iop, sizeof(int) * table_count,
@@ -157,11 +167,11 @@ int spider_udf_direct_sql_create_table_list(
       tmp_name_ptr += length + 1;
       tmp_ptr = tmp_ptr3 + 1;
     } else {
-      if (thd->db)
+      if (SPIDER_THD_db_str(thd))
       {
-        memcpy(tmp_name_ptr, thd->db,
-          thd->db_length + 1);
-        tmp_name_ptr += thd->db_length + 1;
+        memcpy(tmp_name_ptr, SPIDER_THD_db_str(thd),
+          SPIDER_THD_db_length(thd) + 1);
+        tmp_name_ptr += SPIDER_THD_db_length(thd) + 1;
       } else {
         direct_sql->db_names[roop_count] = (char *) "";
       }
@@ -349,6 +359,27 @@ int spider_udf_direct_sql_create_conn_key(
 #endif
     }
   }
+  if (direct_sql->dbton_id == SPIDER_DBTON_SIZE)
+  {
+#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
+    if (direct_sql->access_mode == 0)
+    {
+#endif
+      my_printf_error(
+        ER_SPIDER_SQL_WRAPPER_IS_INVALID_NUM,
+        ER_SPIDER_SQL_WRAPPER_IS_INVALID_STR,
+        MYF(0), direct_sql->tgt_wrapper);
+      DBUG_RETURN(ER_SPIDER_SQL_WRAPPER_IS_INVALID_NUM);
+#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
+    } else {
+      my_printf_error(
+        ER_SPIDER_NOSQL_WRAPPER_IS_INVALID_NUM,
+        ER_SPIDER_NOSQL_WRAPPER_IS_INVALID_STR,
+        MYF(0), direct_sql->tgt_wrapper);
+      DBUG_RETURN(ER_SPIDER_NOSQL_WRAPPER_IS_INVALID_NUM);
+    }
+#endif
+  }
 #ifdef SPIDER_HAS_HASH_VALUE_TYPE
   direct_sql->conn_key_hash_value = my_calc_hash(&spider_open_connections,
     (uchar*) direct_sql->conn_key, direct_sql->conn_key_length);
@@ -361,12 +392,20 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
   int *error_num
 ) {
   SPIDER_CONN *conn;
+  SPIDER_IP_PORT_CONN *ip_port_conn;
   char *tmp_name, *tmp_host, *tmp_username, *tmp_password, *tmp_socket;
   char *tmp_wrapper, *tmp_ssl_ca, *tmp_ssl_capath, *tmp_ssl_cert;
   char *tmp_ssl_cipher, *tmp_ssl_key, *tmp_default_file, *tmp_default_group;
   int *need_mon;
   DBUG_ENTER("spider_udf_direct_sql_create_conn");
 
+  if (unlikely(!UTC))
+  {
+    /* UTC time zone for timestamp columns */
+    String tz_00_name(STRING_WITH_LEN("+00:00"), &my_charset_bin);
+    UTC = my_tz_find(current_thd, &tz_00_name);
+  }
+
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   if (direct_sql->access_mode == 0)
   {
@@ -558,12 +597,57 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
     goto error;
   conn->ping_time = (time_t) time((time_t*) 0);
   conn->connect_error_time = conn->ping_time;
+  pthread_mutex_lock(&spider_conn_id_mutex);
+  conn->conn_id = spider_conn_id;
+  ++spider_conn_id;
+  pthread_mutex_unlock(&spider_conn_id_mutex);
+
+  pthread_mutex_lock(&spider_ipport_conn_mutex);
+#ifdef SPIDER_HAS_HASH_VALUE_TYPE
+  if ((ip_port_conn = (SPIDER_IP_PORT_CONN*) my_hash_search_using_hash_value(
+    &spider_ipport_conns, conn->conn_key_hash_value,
+    (uchar*)conn->conn_key, conn->conn_key_length)))
+#else
+  if ((ip_port_conn = (SPIDER_IP_PORT_CONN*) my_hash_search(
+    &spider_ipport_conns, (uchar*)conn->conn_key, conn->conn_key_length)))
+#endif
+  { /* exists, +1 */
+    pthread_mutex_unlock(&spider_ipport_conn_mutex);
+    pthread_mutex_lock(&ip_port_conn->mutex);
+    if (spider_param_max_connections())
+    { /* enable conncetion pool */
+      if (ip_port_conn->ip_port_count >= spider_param_max_connections())
+      { /* bigger than the max num of connections, free conn and return NULL */
+        pthread_mutex_unlock(&ip_port_conn->mutex);
+        goto error_too_many_ipport_count;
+      }
+    }
+    ip_port_conn->ip_port_count++;
+    pthread_mutex_unlock(&ip_port_conn->mutex);
+  }
+  else
+  {// do not exist
+    ip_port_conn = spider_create_ipport_conn(conn);
+    if (!ip_port_conn) {
+      /* failed, always do not effect 'create conn' */
+      pthread_mutex_unlock(&spider_ipport_conn_mutex);
+      DBUG_RETURN(conn);
+    }
+    if (my_hash_insert(&spider_ipport_conns, (uchar *)ip_port_conn)) {
+      /* insert failed, always do not effect 'create conn' */
+      pthread_mutex_unlock(&spider_ipport_conn_mutex);
+      DBUG_RETURN(conn);
+    }
+    pthread_mutex_unlock(&spider_ipport_conn_mutex);
+  }
+  conn->ip_port_conn = ip_port_conn;
 
   DBUG_RETURN(conn);
 
 error:
   DBUG_ASSERT(!conn->mta_conn_mutex_file_pos.file_name);
   pthread_mutex_destroy(&conn->mta_conn_mutex);
+error_too_many_ipport_count:
 error_mta_conn_mutex_init:
 error_db_conn_init:
   delete conn->db_conn;
@@ -900,13 +984,11 @@ error:
     if (!direct_sql->param_name) \
     { \
       if ((direct_sql->param_name = spider_get_string_between_quote( \
-        start_ptr, TRUE))) \
+        start_ptr, TRUE, &param_string_parse))) \
         direct_sql->SPIDER_PARAM_STR_LEN(param_name) = \
           strlen(direct_sql->param_name); \
       else { \
-        error_num = ER_SPIDER_INVALID_CONNECT_INFO_NUM; \
-        my_printf_error(error_num, ER_SPIDER_INVALID_CONNECT_INFO_STR, \
-          MYF(0), tmp_ptr); \
+        error_num = param_string_parse.print_param_error(); \
         goto error; \
       } \
       DBUG_PRINT("info",("spider " title_name "=%s", direct_sql->param_name)); \
@@ -926,9 +1008,7 @@ error:
     { \
       if (hint_num < 0 || hint_num >= max_size) \
       { \
-        error_num = ER_SPIDER_INVALID_CONNECT_INFO_NUM; \
-        my_printf_error(error_num, ER_SPIDER_INVALID_CONNECT_INFO_STR, \
-          MYF(0), tmp_ptr); \
+        error_num = param_string_parse.print_param_error(); \
         goto error; \
       } else if (direct_sql->param_name[hint_num] != -1) \
         break; \
@@ -941,17 +1021,13 @@ error:
         else if (direct_sql->param_name[hint_num] > max_val) \
           direct_sql->param_name[hint_num] = max_val; \
       } else { \
-        error_num = ER_SPIDER_INVALID_CONNECT_INFO_NUM; \
-        my_printf_error(error_num, ER_SPIDER_INVALID_CONNECT_INFO_STR, \
-          MYF(0), tmp_ptr); \
+        error_num = param_string_parse.print_param_error(); \
         goto error; \
       } \
       DBUG_PRINT("info",("spider " title_name "[%d]=%d", hint_num, \
         direct_sql->param_name[hint_num])); \
     } else { \
-      error_num = ER_SPIDER_INVALID_CONNECT_INFO_NUM; \
-      my_printf_error(error_num, ER_SPIDER_INVALID_CONNECT_INFO_STR, \
-        MYF(0), tmp_ptr); \
+      error_num = param_string_parse.print_param_error(); \
       goto error; \
     } \
     break; \
@@ -970,10 +1046,11 @@ error:
           direct_sql->param_name = min_val; \
         else if (direct_sql->param_name > max_val) \
           direct_sql->param_name = max_val; \
+        param_string_parse.set_param_value(tmp_ptr2, \
+                                           tmp_ptr2 + \
+                                             strlen(tmp_ptr2) + 1); \
       } else { \
-        error_num = ER_SPIDER_INVALID_CONNECT_INFO_NUM; \
-        my_printf_error(error_num, ER_SPIDER_INVALID_CONNECT_INFO_STR, \
-          MYF(0), tmp_ptr); \
+        error_num = param_string_parse.print_param_error(); \
         goto error; \
       } \
       DBUG_PRINT("info",("spider " title_name "=%d", \
@@ -993,10 +1070,11 @@ error:
         direct_sql->param_name = atoi(tmp_ptr2); \
         if (direct_sql->param_name < min_val) \
           direct_sql->param_name = min_val; \
+        param_string_parse.set_param_value(tmp_ptr2, \
+                                           tmp_ptr2 + \
+                                             strlen(tmp_ptr2) + 1); \
       } else { \
-        error_num = ER_SPIDER_INVALID_CONNECT_INFO_NUM; \
-        my_printf_error(error_num, ER_SPIDER_INVALID_CONNECT_INFO_STR, \
-          MYF(0), tmp_ptr); \
+        error_num = param_string_parse.print_param_error(); \
         goto error; \
       } \
       DBUG_PRINT("info",("spider " title_name "=%d", direct_sql->param_name)); \
@@ -1016,10 +1094,11 @@ error:
           my_strtoll10(tmp_ptr2, (char**) NULL, &error_num); \
         if (direct_sql->param_name < min_val) \
           direct_sql->param_name = min_val; \
+        param_string_parse.set_param_value(tmp_ptr2, \
+                                           tmp_ptr2 + \
+                                             strlen(tmp_ptr2) + 1); \
       } else { \
-        error_num = ER_SPIDER_INVALID_CONNECT_INFO_NUM; \
-        my_printf_error(error_num, ER_SPIDER_INVALID_CONNECT_INFO_STR, \
-          MYF(0), tmp_ptr); \
+        error_num = param_string_parse.print_param_error(); \
         goto error; \
       } \
       DBUG_PRINT("info",("spider " title_name "=%lld", \
@@ -1039,6 +1118,7 @@ int spider_udf_parse_direct_sql_param(
   char *sprit_ptr[2];
   char *tmp_ptr, *tmp_ptr2, *start_ptr;
   int title_length;
+  SPIDER_PARAM_STRING_PARSE param_string_parse;
   DBUG_ENTER("spider_udf_parse_direct_sql_param");
   direct_sql->tgt_port = -1;
   direct_sql->tgt_ssl_vsc = -1;
@@ -1075,6 +1155,7 @@ int spider_udf_parse_direct_sql_param(
   DBUG_PRINT("info",("spider param_string=%s", param_string));
 
   sprit_ptr[0] = param_string;
+  param_string_parse.init(param_string, ER_SPIDER_INVALID_UDF_PARAM_NUM);
   while (sprit_ptr[0])
   {
     if ((sprit_ptr[1] = strchr(sprit_ptr[0], ',')))
@@ -1101,10 +1182,14 @@ int spider_udf_parse_direct_sql_param(
       title_length++;
       start_ptr++;
     }
+    param_string_parse.set_param_title(tmp_ptr, tmp_ptr + title_length);
 
     switch (title_length)
     {
       case 0:
+        error_num = param_string_parse.print_param_error();
+        if (error_num)
+          goto error;
         continue;
       case 3:
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
@@ -1130,120 +1215,92 @@ int spider_udf_parse_direct_sql_param(
         SPIDER_PARAM_INT_WITH_MAX("urt", use_real_table, 0, 1);
 #endif
         SPIDER_PARAM_INT("wto", net_write_timeout, 0);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 4:
         SPIDER_PARAM_INT_WITH_MAX("erwm", error_rw_mode, 0, 1);
         SPIDER_PARAM_STR("host", tgt_host);
         SPIDER_PARAM_INT_WITH_MAX("port", tgt_port, 0, 65535);
         SPIDER_PARAM_STR("user", tgt_username);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 6:
         SPIDER_PARAM_STR("server", server_name);
         SPIDER_PARAM_STR("socket", tgt_socket);
         SPIDER_PARAM_HINT_WITH_MAX("iop", iop, 3, direct_sql->table_count, 0, 2);
         SPIDER_PARAM_STR("ssl_ca", tgt_ssl_ca);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 7:
         SPIDER_PARAM_STR("wrapper", tgt_wrapper);
         SPIDER_PARAM_STR("ssl_key", tgt_ssl_key);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 8:
         SPIDER_PARAM_STR("database", tgt_default_db_name);
         SPIDER_PARAM_STR("password", tgt_password);
         SPIDER_PARAM_LONGLONG("priority", priority, 0);
         SPIDER_PARAM_STR("ssl_cert", tgt_ssl_cert);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 10:
         SPIDER_PARAM_STR("ssl_cipher", tgt_ssl_cipher);
         SPIDER_PARAM_STR("ssl_capath", tgt_ssl_capath);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 11:
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
         SPIDER_PARAM_INT_WITH_MAX("access_mode", access_mode, 0, 2);
 #endif
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 12:
         SPIDER_PARAM_STR("default_file", tgt_default_file);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 13:
         SPIDER_PARAM_STR("default_group", tgt_default_group);
         SPIDER_PARAM_INT_WITH_MAX("error_rw_mode", error_rw_mode, 0, 1);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 14:
 #if MYSQL_VERSION_ID < 50500
 #else
         SPIDER_PARAM_INT_WITH_MAX("use_real_table", use_real_table, 0, 1);
 #endif
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 15:
         SPIDER_PARAM_INT_WITH_MAX("table_loop_mode", table_loop_mode, 0, 2);
         SPIDER_PARAM_INT("connect_timeout", connect_timeout, 0);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 16:
         SPIDER_PARAM_LONGLONG("bulk_insert_rows", bulk_insert_rows, 1);
         SPIDER_PARAM_INT("net_read_timeout", net_read_timeout, 0);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 17:
         SPIDER_PARAM_INT("net_write_timeout", net_write_timeout, 0);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 18:
         SPIDER_PARAM_INT_WITH_MAX(
           "connection_channel", connection_channel, 0, 63);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       case 22:
         SPIDER_PARAM_INT_WITH_MAX("ssl_verify_server_cert", tgt_ssl_vsc, 0, 1);
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
       default:
-        error_num = ER_SPIDER_INVALID_UDF_PARAM_NUM;
-        my_printf_error(error_num, ER_SPIDER_INVALID_UDF_PARAM_STR,
-          MYF(0), tmp_ptr);
+        error_num = param_string_parse.print_param_error();
         goto error;
     }
+
+    /* Verify that the remainder of the parameter value is whitespace */
+    if ((error_num = param_string_parse.has_extra_parameter_values()))
+      goto error;
   }
 
 set_default:
@@ -1283,10 +1340,10 @@ int spider_udf_set_direct_sql_param_default(
   if (!direct_sql->tgt_default_db_name)
   {
     DBUG_PRINT("info",("spider create default tgt_default_db_name"));
-    direct_sql->tgt_default_db_name_length = trx->thd->db_length;
+    direct_sql->tgt_default_db_name_length = SPIDER_THD_db_length(trx->thd);
     if (
       !(direct_sql->tgt_default_db_name = spider_create_string(
-        trx->thd->db,
+        SPIDER_THD_db_str(trx->thd),
         direct_sql->tgt_default_db_name_length))
     ) {
       my_error(ER_OUT_OF_RESOURCES, MYF(0), HA_ERR_OUT_OF_MEM);
@@ -1622,6 +1679,15 @@ long long spider_direct_sql_body(
       goto error;
     }
   }
+#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
+  if (trx->trx_start && direct_sql->access_mode != 1)
+  {
+#endif
+    trx->updated_in_this_trx = TRUE;
+    DBUG_PRINT("info",("spider trx->updated_in_this_trx=TRUE"));
+#if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
+  }
+#endif
 #if MYSQL_VERSION_ID < 50500
 #else
   use_real_table = spider_param_udf_ds_use_real_table(thd,
@@ -1630,17 +1696,32 @@ long long spider_direct_sql_body(
   for (roop_count = 0; roop_count < direct_sql->table_count; roop_count++)
   {
 #ifdef SPIDER_NEED_INIT_ONE_TABLE_FOR_FIND_TEMPORARY_TABLE
+#ifdef SPIDER_use_LEX_CSTRING_for_database_tablename_alias
+    LEX_CSTRING db_name =
+    {
+      direct_sql->db_names[roop_count],
+      strlen(direct_sql->db_names[roop_count])
+    };
+    LEX_CSTRING tbl_name =
+    {
+      direct_sql->table_names[roop_count],
+      strlen(direct_sql->table_names[roop_count])
+    };
+    table_list.init_one_table(&db_name, &tbl_name, 0, TL_WRITE);
+#else
     table_list.init_one_table(direct_sql->db_names[roop_count],
       strlen(direct_sql->db_names[roop_count]),
       direct_sql->table_names[roop_count],
       strlen(direct_sql->table_names[roop_count]),
       direct_sql->table_names[roop_count], TL_WRITE);
+#endif
 #else
-    table_list.db = direct_sql->db_names[roop_count];
-    table_list.table_name = direct_sql->table_names[roop_count];
+    SPIDER_TABLE_LIST_db_str(&table_list) = direct_sql->db_names[roop_count];
+    SPIDER_TABLE_LIST_table_name_str(&table_list) =
+      direct_sql->table_names[roop_count];
 #endif
     if (!(direct_sql->tables[roop_count] =
-      find_temporary_table(thd, &table_list)))
+      spider_find_temporary_table(thd, &table_list)))
     {
 #if MYSQL_VERSION_ID < 50500
 #else
@@ -1650,17 +1731,28 @@ long long spider_direct_sql_body(
         error_num = ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_NUM;
         my_printf_error(ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_NUM,
           ER_SPIDER_UDF_TMP_TABLE_NOT_FOUND_STR,
-          MYF(0), table_list.db, table_list.table_name);
+          MYF(0), SPIDER_TABLE_LIST_db_str(&table_list),
+          SPIDER_TABLE_LIST_table_name_str(&table_list));
         goto error;
 #if MYSQL_VERSION_ID < 50500
 #else
       }
       TABLE_LIST *tables = &direct_sql->table_list[roop_count];
-      tables->init_one_table(table_list.db, strlen(table_list.db),
-        table_list.table_name, strlen(table_list.table_name),
-        table_list.table_name, TL_WRITE);
-      tables->mdl_request.init(MDL_key::TABLE, table_list.db,
-        table_list.table_name, MDL_SHARED_WRITE, MDL_TRANSACTION);
+#ifdef SPIDER_use_LEX_CSTRING_for_database_tablename_alias
+      table_list.init_one_table(
+        &table_list.db, &table_list.table_name, 0, TL_WRITE);
+#else
+      tables->init_one_table(
+        SPIDER_TABLE_LIST_db_str(&table_list),
+        SPIDER_TABLE_LIST_db_length(&table_list),
+        SPIDER_TABLE_LIST_table_name_str(&table_list),
+        SPIDER_TABLE_LIST_table_name_length(&table_list),
+        SPIDER_TABLE_LIST_table_name_str(&table_list), TL_WRITE);
+#endif
+      tables->mdl_request.init(MDL_key::TABLE,
+        SPIDER_TABLE_LIST_db_str(&table_list),
+        SPIDER_TABLE_LIST_table_name_str(&table_list),
+        MDL_SHARED_WRITE, MDL_TRANSACTION);
       if (!direct_sql->table_list_first)
       {
         direct_sql->table_list_first = tables;
