@@ -60,13 +60,13 @@ extern "C" UINT __stdcall RemoveDataDirectory(MSIHANDLE hInstall)
 {
   HRESULT hr = S_OK;
   UINT er = ERROR_SUCCESS;
+  wchar_t dir[MAX_PATH];
+  DWORD len = MAX_PATH;
 
   hr = WcaInitialize(hInstall, __FUNCTION__);
   ExitOnFailure(hr, "Failed to initialize");
   WcaLog(LOGMSG_STANDARD, "Initialized.");
 
-  wchar_t dir[MAX_PATH];
-  DWORD len = MAX_PATH;
   MsiGetPropertyW(hInstall, L"CustomActionData", dir, &len);
 
   er= ExecRemoveDataDirectory(dir);
@@ -90,7 +90,7 @@ static void EscapeCommandLine(const wchar_t *in, wchar_t *out, size_t buflen)
   bool needs_escaping= false;
   size_t pos;
 
-  for(int i=0; i< sizeof(special_chars) -1; i++)
+  for(size_t i=0; i< sizeof(special_chars) -1; i++)
   {
     if (wcschr(in, special_chars[i]))
     {
@@ -160,20 +160,21 @@ extern "C" UINT __stdcall CheckDirectoryEmpty(MSIHANDLE hInstall,
 {
   HRESULT hr = S_OK;
   UINT er = ERROR_SUCCESS;
+  wchar_t buf[MAX_PATH];
+  DWORD len = MAX_PATH;
+  WIN32_FIND_DATAW data;
+  HANDLE h;
+  bool empty;
+  
   hr = WcaInitialize(hInstall, __FUNCTION__);
   ExitOnFailure(hr, "Failed to initialize");
   WcaLog(LOGMSG_STANDARD, "Initialized.");
 
-  
-  wchar_t buf[MAX_PATH];
-  DWORD len = MAX_PATH;
   MsiGetPropertyW(hInstall, PropertyName, buf, &len);
   wcscat_s(buf, MAX_PATH, L"*.*");
   
   WcaLog(LOGMSG_STANDARD, "Checking files in %S", buf);
-  WIN32_FIND_DATAW data;
-  HANDLE h;
-  bool empty;
+
   h= FindFirstFile(buf, &data);
   if (h !=  INVALID_HANDLE_VALUE)
   {
@@ -270,7 +271,7 @@ bool IsPortFree(short port)
   struct sockaddr_in sin;
   SOCKET sock;
   sock = socket(AF_INET, SOCK_STREAM, 0);
-  if(sock == -1)
+  if(sock == INVALID_SOCKET)
   {
     return false;
   }
@@ -342,6 +343,12 @@ void CheckServiceConfig(
   wchar_t * commandline= config->lpBinaryPathName;
   int numargs;
   wchar_t **argv= CommandLineToArgvW(commandline, &numargs);
+  wchar_t current_datadir_buf[MAX_PATH]={0};
+  wchar_t normalized_current_datadir[MAX_PATH+1];
+  wchar_t *current_datadir;
+  wchar_t *defaults_file;
+  bool is_my_service;
+
   WcaLog(LOGMSG_VERBOSE, "CommandLine= %S", commandline);
   if(!argv  ||  !argv[0]  || ! wcsstr(argv[0], L"mysqld"))
   {
@@ -356,7 +363,7 @@ void CheckServiceConfig(
     same_bindir = true;
   }
 
-  bool is_my_service = (_wcsicmp(my_servicename, other_servicename) == 0);
+  is_my_service = (_wcsicmp(my_servicename, other_servicename) == 0);
   if(!is_my_service)
   {
     WcaLog(LOGMSG_STANDARD, "service does not match current service");
@@ -379,10 +386,8 @@ void CheckServiceConfig(
     goto end;
   }
 
-  wchar_t current_datadir_buf[MAX_PATH]={0};
-  wchar_t normalized_current_datadir[MAX_PATH+1];
-  wchar_t *current_datadir= current_datadir_buf;
-  wchar_t *defaults_file= argv[1]+16;
+  current_datadir= current_datadir_buf;
+  defaults_file= argv[1]+16;
   defaults_file= strip_quotes(defaults_file);
 
   WcaLog(LOGMSG_STANDARD, "parsed defaults file is %S", defaults_file);
@@ -453,6 +458,7 @@ extern "C" UINT CheckDBInUse(MSIHANDLE hInstall)
   ULONG  bufneed   = 0x00; 
   ULONG  num_services = 0x00; 
   LPENUM_SERVICE_STATUS_PROCESS info = NULL;  
+  BOOL ok;
 
   hr = WcaInitialize(hInstall, __FUNCTION__);
   ExitOnFailure(hr, "Failed to initialize");
@@ -471,7 +477,7 @@ extern "C" UINT CheckDBInUse(MSIHANDLE hInstall)
     ExitOnFailure(E_FAIL, "OpenSCManager failed");
   }
 
-  BOOL ok= EnumServicesStatusExW(  scm,
+  ok = EnumServicesStatusExW(  scm,
     SC_ENUM_PROCESS_INFO, 
     SERVICE_WIN32,
     SERVICE_STATE_ALL,  
@@ -524,7 +530,7 @@ LExit:
   or 4GB(x64 OS).
   
   Fragmentation due to loaded modules, heap  and stack 
-  limit maximum size of continous memory block further,
+  limit maximum size of continuous memory block further,
   so that limit for 32 bit process is about 1200 on 32 bit OS 
   or 2000 MB on 64 bit OS(found experimentally).
 */
@@ -559,13 +565,16 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
   const wchar_t *ErrorMsg=0;
   HRESULT hr= S_OK;
   UINT er= ERROR_SUCCESS;
-
+  DWORD ServiceNameLen = MAX_PATH;
+  DWORD QuickConfigLen = MAX_PATH;
+  DWORD PasswordLen= MAX_PATH;
+  DWORD SkipNetworkingLen= MAX_PATH;
 
   hr = WcaInitialize(hInstall, __FUNCTION__);
   ExitOnFailure(hr, "Failed to initialize");
   WcaLog(LOGMSG_STANDARD, "Initialized.");
 
-  DWORD ServiceNameLen = MAX_PATH;
+ 
   MsiGetPropertyW (hInstall, L"SERVICENAME", ServiceName, &ServiceNameLen);
   if(ServiceName[0])
   {
@@ -594,13 +603,11 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
     }
   }
 
-  DWORD PasswordLen= MAX_PATH;
   MsiGetPropertyW (hInstall, L"PASSWORD", Password, &PasswordLen);
   EscapeCommandLine(Password, EscapedPassword,
     sizeof(EscapedPassword)/sizeof(EscapedPassword[0]));
   MsiSetPropertyW(hInstall,L"ESCAPEDPASSWORD",EscapedPassword);
 
-  DWORD SkipNetworkingLen= MAX_PATH;
   MsiGetPropertyW(hInstall, L"SKIPNETWORKING", SkipNetworking, 
     &SkipNetworkingLen);
   MsiGetPropertyW(hInstall, L"PORT", Port, &PortLen);
@@ -644,8 +651,6 @@ extern "C" UINT  __stdcall CheckDatabaseProperties (MSIHANDLE hInstall)
     }
   }
   
-  
-  DWORD QuickConfigLen = MAX_PATH;
   MsiGetPropertyW (hInstall, L"STDCONFIG", QuickConfig, &QuickConfigLen);
   if(QuickConfig[0] !=0)
   {
@@ -722,12 +727,12 @@ extern "C" UINT __stdcall PresetDatabaseProperties(MSIHANDLE hInstall)
   UINT er = ERROR_SUCCESS;
   HRESULT hr= S_OK;
   MEMORYSTATUSEX memstatus;
+  DWORD BufferPoolsizeParamLen = MAX_PATH;
   hr = WcaInitialize(hInstall, __FUNCTION__);
   ExitOnFailure(hr, "Failed to initialize");
   WcaLog(LOGMSG_STANDARD, "Initialized.");
 
   /* Check if bufferpoolsize parameter was given on the command line*/
-  DWORD BufferPoolsizeParamLen = MAX_PATH;
   MsiGetPropertyW(hInstall, L"BUFFERPOOLSIZE", buff, &BufferPoolsizeParamLen);
 
   if (BufferPoolsizeParamLen && buff[0])
@@ -767,6 +772,49 @@ extern "C" UINT __stdcall PresetDatabaseProperties(MSIHANDLE hInstall)
 LExit:
   return WcaFinalize(er);
 }
+
+static BOOL FindErrorLog(const wchar_t *dir, wchar_t * ErrorLogFile, size_t ErrorLogLen)
+{
+  WIN32_FIND_DATA FindFileData;
+  HANDLE hFind;
+  wchar_t name[MAX_PATH];
+  wcsncpy_s(name,dir, MAX_PATH);
+  wcsncat_s(name,L"\\*.err", MAX_PATH);
+  hFind = FindFirstFileW(name,&FindFileData);
+  if (hFind != INVALID_HANDLE_VALUE)
+  {
+    _snwprintf(ErrorLogFile, ErrorLogLen,
+      L"%s\\%s",dir, FindFileData.cFileName);
+    FindClose(hFind);
+    return TRUE;
+  }
+  return FALSE;
+}
+
+static void DumpErrorLog(const wchar_t *dir)
+{
+  wchar_t filepath[MAX_PATH];
+  if (!FindErrorLog(dir, filepath, MAX_PATH))
+    return;
+  FILE *f= _wfopen(filepath, L"r");
+  if (!f)
+    return;
+  char buf[2048];
+  WcaLog(LOGMSG_STANDARD,"=== dumping error log %S === ",filepath);
+  while (fgets(buf, sizeof(buf), f))
+  {
+     /* Strip off EOL chars. */
+     size_t len = strlen(buf);
+     if (len > 0 && buf[len-1] == '\n')
+       buf[--len]= 0;
+     if (len > 0 && buf[len-1] == '\r')
+       buf[--len]= 0;
+     WcaLog(LOGMSG_STANDARD,"%s",buf);
+  }
+  fclose(f);
+  WcaLog(LOGMSG_STANDARD,"=== end of error log ===");
+}
+
 /* Remove service and data directory created by CreateDatabase operation */
 extern "C" UINT __stdcall CreateDatabaseRollback(MSIHANDLE hInstall) 
 {
@@ -774,12 +822,13 @@ extern "C" UINT __stdcall CreateDatabaseRollback(MSIHANDLE hInstall)
   UINT er = ERROR_SUCCESS;
   wchar_t* service= 0;
   wchar_t* dir= 0;
+  wchar_t data[2*MAX_PATH];
+  DWORD len= MAX_PATH;
 
   hr = WcaInitialize(hInstall, __FUNCTION__);
   ExitOnFailure(hr, "Failed to initialize");
   WcaLog(LOGMSG_STANDARD, "Initialized.");
-  wchar_t data[2*MAX_PATH];
-  DWORD len= MAX_PATH;
+
   MsiGetPropertyW(hInstall, L"CustomActionData", data, &len);
 
   /* Property is encoded as [SERVICENAME]\[DBLOCATION] */
@@ -804,6 +853,7 @@ extern "C" UINT __stdcall CreateDatabaseRollback(MSIHANDLE hInstall)
   }
   if(dir)
   {
+    DumpErrorLog(dir);
     ExecRemoveDataDirectory(dir);
   }
 LExit:
@@ -821,13 +871,16 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
 {
   HRESULT hr = S_OK;
   UINT er = ERROR_SUCCESS;
-  wchar_t* service= 0;
-  wchar_t* dir= 0;
   wchar_t installerVersion[MAX_VERSION_PROPERTY_SIZE];
   char installDir[MAX_PATH];
   DWORD size =MAX_VERSION_PROPERTY_SIZE;
   int installerMajorVersion, installerMinorVersion, installerPatchVersion;
   bool upgradableServiceFound=false;
+  LPENUM_SERVICE_STATUS_PROCESSW info;
+  DWORD bufsize;
+  int index;
+  BOOL ok;
+  SC_HANDLE scm = NULL;
 
   hr = WcaInitialize(hInstall, __FUNCTION__);
    WcaLog(LOGMSG_STANDARD, "Initialized.");
@@ -852,7 +905,7 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
   }
  
 
-  SC_HANDLE scm = OpenSCManager(NULL, NULL, 
+  scm = OpenSCManager(NULL, NULL, 
     SC_MANAGER_ENUMERATE_SERVICE | SC_MANAGER_CONNECT);  
   if (scm == NULL) 
   { 
@@ -863,19 +916,19 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
   static BYTE buf[64*1024];
   static BYTE config_buffer[8*1024];
 
-  DWORD bufsize= sizeof(buf);
+  bufsize= sizeof(buf);
   DWORD bufneed;
   DWORD num_services;
-  BOOL ok= EnumServicesStatusExW(scm, SC_ENUM_PROCESS_INFO,  SERVICE_WIN32,
+  ok= EnumServicesStatusExW(scm, SC_ENUM_PROCESS_INFO,  SERVICE_WIN32,
     SERVICE_STATE_ALL,  buf, bufsize,  &bufneed, &num_services, NULL, NULL);
   if(!ok) 
   {
     hr = HRESULT_FROM_WIN32(GetLastError());
     ExitOnFailure(hr,"EnumServicesStatusEx failed");
   }
-  LPENUM_SERVICE_STATUS_PROCESSW info =
+  info =
     (LPENUM_SERVICE_STATUS_PROCESSW)buf;
-  int index=-1;
+  index=-1;
   for (ULONG i=0; i < num_services; i++)
   {
     SC_HANDLE service= OpenServiceW(scm, info[i].lpServiceName, 
@@ -885,7 +938,7 @@ extern "C" UINT __stdcall CheckServiceUpgrades(MSIHANDLE hInstall)
     QUERY_SERVICE_CONFIGW *config= 
       (QUERY_SERVICE_CONFIGW*)(void *)config_buffer;
     DWORD needed;
-    BOOL ok= QueryServiceConfigW(service, config,sizeof(config_buffer),
+    ok= QueryServiceConfigW(service, config,sizeof(config_buffer),
       &needed) && (config->dwStartType != SERVICE_DISABLED);
     CloseServiceHandle(service);
     if (ok)
